package textrecognitionwrapper;


import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.util.DisplayMetrics;
import android.view.ViewGroup;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Permissions;

import anywheresoftware.b4a.BA.Events;

import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.SurfaceHolder.Callback;

import com.camerasource.CameraSource;

import com.google.android.gms.vision.Detector;
import com.google.android.gms.vision.text.TextBlock;
import com.google.android.gms.vision.text.Text;
import com.google.android.gms.vision.text.TextRecognizer;
import android.util.SparseArray;

import android.widget.TextView;
import android.view.View;
import android.view.MotionEvent;
import java.util.List; 
import java.util.ArrayList; 

import java.io.IOException;

import android.hardware.Camera;
import java.lang.reflect.Field;

@ActivityObject
@Events(values={"text_scanned_blocks(scannedBlocks As Object)", "text_scanned_lines(scannedLines As Object)", "text_scanned_words(scannedWords As Object)"})
@Author("Wrapped by: Johan Schoeman")
@Version(1.01f)
@ShortName("AnroidTextRecognition")
@DependsOn(values={"com.google.android.gms:play-services-vision", "com.android.support:appcompat-v7"})
@Permissions(values={"android.permission.CAMERA", "android.permission.FLASHLIGHT"})

public class textrecognitionWrapper extends ViewWrapper<SurfaceView> implements DesignerCustomView {


	private BA ba;
	private String eventName;
    private SurfaceView cv;
	CameraSource mCameraSource;
	TextView mTextView;
	TextRecognizer textRecognizer;
	String blocks;
	String lines;
	String words;

	private ArrayList<String> scannedBlocks;
	private ArrayList<String> scannedLines;
	private ArrayList<String> scannedWords;
	public boolean scannerActive = false;
 
	
	 /**
     * 
     * Initialize the Object with the EventName
	 * It will automatically be initialized when added as a custom view via the B4A Designer
     */
	public void Initialize(BA ba, String EventName) {
		_initialize(ba, null, EventName);
	}

	@Hide
	public void _initialize(BA ba, Object activityClass, String EventName) {
		this.eventName = EventName.toLowerCase(BA.cul);
		this.ba = ba;
		cv = new SurfaceView(ba.context);
        this.setObject(cv);

 	}
	
	
	 /**
     * 
     * Start the scanner for OCR
	 * No parameters required
     */
    public void startScanning() {
		startCameraSource();
		try {
			mCameraSource.start(cv.getHolder());
			scannerActive = true;
		} catch (IOException ie) {
			//Log.e("CAMERA SOURCE", ie.getMessage());
		}
	}	
	
	 /**
     * 
     * Stop the OCR scanner
	 * No parameters required
     */	
	public void stopScanning() {
		mCameraSource.release();
        textRecognizer.release();
		scannerActive = false;
	}
	
	

	
	 /**
     * 
     * Returns and Object holding an ArrayList of all Blocks extracted
	 * Assign the Object to a List in B4A
	 * eg Dim blk As List = atr1.TextBlocks 
     */		
	public ArrayList<String> getTextBlocks() {
		if (scannerActive == true) {
			return scannedBlocks;
		} else {
			scannedBlocks = new ArrayList<String>();
			scannedBlocks.add("NIL");
			return scannedBlocks;
		}	
	}
	
	 /**
     * 
     * Returns and Object holding an ArrayList of all Lines extracted
	 * Assign the Object to a List in B4A
	 * eg Dim blk As List = atr1.TextLines 	 
     */			
	public ArrayList<String> getTextLines() {
		if (scannerActive == true) {
			return scannedLines;
		} else {
			scannedLines = new ArrayList<String>();
			scannedLines.add("NIL");
			return scannedLines;
		}	
	}

	 /**
     * 
     * Returns and Object holding an ArrayList of all Words extracted
	 * Assign the Object to a List in B4A
	 * eg Dim blk As List = atr1.TextWords 
     */			
	public ArrayList<String> getTextWords() {
		if (scannerActive == true) {
			return scannedWords;
		} else {
			scannedWords = new ArrayList<String>();
			scannedWords.add("NIL");
			return scannedWords;
		}	
	}	
	
	 /**
     * 
     * Turn the flash on
	 * Will only function if the OCR scanner is active
     */			
	public void setFlashOn() {
		if (mCameraSource != null) {
		  mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
		}
	}
	
	 /**
     * 
     * Turn the flash off
	 * Will only function if the OCR scanner is active
     */		
	public void setFlashOff() {
		if (mCameraSource != null) {
		  mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
		}
	}

	
	
    private void startCameraSource() {
		

        //Create the TextRecognizer
        textRecognizer = new TextRecognizer.Builder(ba.context).build();

        if (!textRecognizer.isOperational()) {
            BA.Log("Detector dependencies not loaded yet");
        } else {

            //Initialize camerasource to use high resolution and set Autofocus on.
            mCameraSource = new CameraSource.Builder(ba.context, textRecognizer)
                    .setFacing(CameraSource.CAMERA_FACING_BACK)
                    .setRequestedPreviewSize(1280, 1024)
                    .setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_VIDEO)
                    .setRequestedFps(2.0f)
                    .build();


            /**
             * Add call back to SurfaceView and check if camera permission is granted.
             * If permission is granted we can start our cameraSource and pass it to surfaceView
             */
            cv.getHolder().addCallback(new SurfaceHolder.Callback() {
                @Override
                public void surfaceCreated(SurfaceHolder holder) {
					
                }

                @Override
                public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
					//BA.Log("Surface created");
                }

                @Override
                public void surfaceDestroyed(SurfaceHolder holder) {
                    mCameraSource.stop();
					mCameraSource = null;
					scannerActive = false;
					//BA.Log("Surface destroyed");
                }
            });
			
			cv.setOnTouchListener(new View.OnTouchListener() {
				@Override
				public boolean onTouch(View v, MotionEvent event) {
					int maskedAction = event.getActionMasked();
					if (maskedAction == MotionEvent.ACTION_UP) {
						//BA.Log("UP");
						return true;
					} else {
						if (ba.subExists(eventName + "_text_scanned_blocks")) {
							ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_text_scanned_blocks", true, new Object[]{scannedBlocks});
						}		

						if (ba.subExists(eventName + "_text_scanned_lines")) {
							ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_text_scanned_lines", true, new Object[]{scannedLines});
						}

						if (ba.subExists(eventName + "_text_scanned_words")) {
							ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_text_scanned_words", true, new Object[]{scannedWords});
						}						
						return false;
						
					}	
					
				}
			});
					
				
            //Set the TextRecognizer's Processor.
            textRecognizer.setProcessor(new Detector.Processor<TextBlock>() {
                @Override
                public void release() {
					
                }

                /**
                 * Detect all the text from camera using TextBlock and the values into a stringBuilder
                 * which will then be set to the textView.
                 * */
                @Override
                public void receiveDetections(Detector.Detections detections) {
					SparseArray items = detections.getDetectedItems();  

					
					scannedBlocks = new ArrayList<String>();
					scannedLines = new ArrayList<String>();
					scannedWords = new ArrayList<String>();
					
					for (int i = 0; i < items.size(); i++)  {  
						TextBlock item = (TextBlock)items.valueAt(i);  
						scannedBlocks.add(item.getValue().toString());
							for (Text line : item.getComponents()) {  
								//extract scanned text lines here  
								scannedLines.add(line.getValue().toString());
								for (Text element : line.getComponents()) {  
									//extract scanned text words here  
									scannedWords.add(element.getValue().toString());
								}  
							}  
					}  
					
					
					
					

                }
            });
        }
    }
	
	
	
	//programmatically add view
    @Hide
	public void AddToParent(ViewGroup Parent, @Pixel int left, @Pixel int top, @Pixel int width, @Pixel int height) {
		Parent.addView(cv, new BALayout.LayoutParams(left, top, width, height));
	}
	
	//this method cannot be hidden.
  
        
	public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, anywheresoftware.b4a.objects.collections.Map props) {
		ViewGroup vg = (ViewGroup) base.getObject().getParent();
		AddToParent(vg, base.getLeft(), base.getTop(), base.getWidth(), base.getHeight());
		base.RemoveView();
		//set text properties
	}

	public void setLeft(int left)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.left = left;
		cv.getParent().requestLayout();
	}

	public int getLeft()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.left;
	}
	public void setTop(int top)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.top = top;
		cv.getParent().requestLayout();
	}
 
	public int getTop()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.top;
	}
	
	public void setWidth(int Width)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.width = Width;
		cv.getParent().requestLayout();
	}

	public int getWidth()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.width;
	}
	
	public void setHeight(int height)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.height = height;
		cv.getParent().requestLayout();
	}

	public int getHeight()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.height;
	}

	
	
   
}
